/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.buttons.alerts');

org.ametys.cms.buttons.alerts.AlertsButton = function(id, configuration, pluginName) 
{
    org.ametys.cms.buttons.alerts.AlertsButton.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.cms.buttons.alerts.AlertsButton, org.ametys.ribbon.control.button.ActionButton, {});

/**
 * @private
 * @property _lastParams The params received on the last refresh
 */
org.ametys.cms.buttons.alerts.AlertsButton.prototype._lastParams;

/**
 * @private
 * @property {String[]} _currentContentsId The content id currently selected or null. 
 */
org.ametys.cms.buttons.alerts.AlertsButton.prototype._currentContentsId;

/**
 * @private
 * @property {String[]} _currentAlertsEnabledContentsId The content id currently selected with alerts enabled or null. 
 */
org.ametys.cms.buttons.alerts.AlertsButton.prototype._currentAlertsEnabledContentsId;

/**
 * @private
 * @property {String[]} _currentAlertsDisabledContentsId The content id currently selected with alerts disabled or null. 
 */
org.ametys.cms.buttons.alerts.AlertsButton.prototype._currentAlertsDisabledContentsId;

/**
 * @private
 * @property {String} _validationAlertDelay the configured delay of the "awaiting validation" alert.
 */
org.ametys.cms.buttons.alerts.AlertsButton.prototype._validationAlertDelay;

/**
 * @private
 * @property {String} _unmodifiedAlertDelay the configured delay of the "unmodified content" alert.
 */
org.ametys.cms.buttons.alerts.AlertsButton.prototype._unmodifiedAlertDelay;

org.ametys.cms.buttons.alerts.AlertsButton.prototype.onMessage = function(messages)
{
    // SELECTION
    for (var i=messages.length - 1; i >= 0; i--)
    {
        if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
        {
            var targets = messages[i].getTargets();
            var allTargetsObject = org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkTarget());
            var allTargets = this._convertToId(allTargetsObject);
            
            // No content selected
            if (allTargets.length == 0)
            {
                this.getElements().each(function(element) { element.setDisabled(true); });
                this.updateTooltip(this.getConfiguration()['default-description']);
                
                this._currentContentsId = [];
            }
            else
            {
                this._currentEdition = allTargetsObject.length == 1 && allTargetsObject[0].getSubTargets().length > 0 && allTargetsObject[0].getSubTargets()[0].getType() == 'form';
                if (!this._areTheSame(this._currentContentsId, allTargets))
                {
                    this._currentContentsId = allTargets;
                    this.getElements().each(function(element) { element.refreshing(); element.setDisabled(true); });
                    this.updateTooltip(this.getConfiguration()['default-description']);
                    var params = {targets: allTargets};
                    this._setRefreshParams(params);
                    org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, params);
                }
                else if (this._currentEdition)
                {
                    // A single element selected, is it for edition ?
                    this.updateTooltip(this.getConfiguration()['editing-description']);
                    this.getElements().each(function(element) { element.setDisabled(true); });
                }
            }
            return;
        }
        else if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_METADATACHANGED
                || messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_MINORMETADATACHANGED)
        {
            var targets = messages[i].getTargets();
            var allTargetsObject = org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkTarget());
            var allTargets = this._convertToId(allTargetsObject);
            
            for (var j = 0; j < allTargets.length; j++)
            {
                if (this._currentContentsId.indexOf(allTargets[j]) >= 0)
                {
                    this.getElements().each(function(element) { element.refreshing(); element.setDisabled(true); });
                    var params = {targets: allTargets};
                    this._setRefreshParams(params);
                    org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, params);
                    
                    return;
                }
            }
        }
    }
}

org.ametys.cms.buttons.alerts.AlertsButton.prototype._setRefreshParams = function(params)
{
}

org.ametys.cms.buttons.alerts.AlertsButton.prototype._areTheSame = function(tab1, tab2)
{
    if (tab1 == tab2)
    {
        return true;
    }

    if (tab1 == null || tab2 == null || tab1.length != tab2.length)
    {
        return false;
    }	
    
    for (var i=0; i < tab1.length; i++)
    {
        if (tab2.indexOf(tab1[i]) == -1)
        {
            return false;
        }
    }
    
    return true;
}

org.ametys.cms.buttons.alerts.AlertsButton.prototype._isInSelection = function(tab, elmt)
{
    if (tab == null || tab.length == 0)
    {
        return false;
    }	
    
    for (var i=0; i < tab.length; i++)
    {
        if (tab[i] == elmt)
        {
            return true;
        }
    }
    
    return false;
    
}

org.ametys.cms.buttons.alerts.AlertsButton.prototype._convertToId = function(targets)
{
    var targetsIds = [];
    for (var i = 0; i < targets.length; i++)
    {
        targetsIds.push(targets[i].getParameters()['id']);
    }
    return targetsIds;
}
org.ametys.cms.buttons.alerts.AlertsButton.prototype.checkTarget = function()
{
    return function(target) { return target.getType() == 'content'; };
}

org.ametys.cms.buttons.alerts.AlertsButton.prototype.configurationError = function(response)
{
    this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(true); });
    this.updateTooltip(this.getConfiguration()['default-description']);
}

org.ametys.cms.buttons.alerts.AlertsButton.prototype.configurationChanged = function(params)
{
    this.getElements().each(function(element) { element.stopRefreshing(); });
    
    this._validationAlertDelay = params['validation-alert-delay'];
    this._unmodifiedAlertDelay = params['unmodified-alert-delay'];
    
    var tooltip = this._getTooltip('', params);
    
    var hasOneContent = tooltip.length > 0; 
    if (!hasOneContent)
    {
        tooltip += this.getConfiguration()['error-description'];
    }
    
    var toggleState = this._isToggle(params);
    this.getElements().each(function(element) { element.allowDepress = false; element.toggle(toggleState); });
    
    if (this._currentEdition)
    {
        this.getElements().each(function(element) { element.setDisabled(true); });
        tooltip = this.getConfiguration()['editing-description'];
    }
    else
    {
        var disabledState = this._isDisabled(params);
        this.getElements().each(function(element) { element.setDisabled(disabledState); });
    }
    
    this.updateTooltip(tooltip, this.getElements().get(0).iconLarge, null);
    
    this._lastParams = params;
}

org.ametys.cms.buttons.alerts.AlertsButton.prototype._isToggle = function(params)
{
    var enabledCount = parseInt(params['alerts-enabled-count']);
    return enabledCount > 0;
}

org.ametys.cms.buttons.alerts.AlertsButton.prototype._isDisabled = function(params)
{
    return false;
}

org.ametys.cms.buttons.alerts.AlertsButton.prototype._getTooltip = function(tooltip, params)
{
    this._currentAlertsEnabledContentsId = [];
    this._currentAlertsDisabledContentsId = [];
    
    var enabledCount = parseInt(params['alerts-enabled-count']);
    var disabledCount = parseInt(params['alerts-disabled-count']);
    
    if (enabledCount > 0)
    {
        if (tooltip != '')
        {
            tooltip += '<br/><br/>';
        }
        
        tooltip += this.getConfiguration()['alerts-enabled-start-description'];
        for (var i=0; i < enabledCount; i++)
        {
            if (i != 0) 
            {
                tooltip += '<br/>';
            }
            tooltip += params['alerts-enabled-description-' + i];
            this._currentAlertsEnabledContentsId.push(params['alerts-enabled-' + i]);
        }
        tooltip += this.getConfiguration()['alerts-enabled-end-description'];
    }
    
    if (disabledCount > 0)
    {
        if (tooltip != '')
        {
            tooltip += '<br/><br/>';
        }
        
        tooltip += this.getConfiguration()['alerts-disabled-start-description'];
        for (var i=0; i < disabledCount; i++)
        {
            if (i != 0) 
            {
                tooltip += ', ';
            }
            tooltip += params['alerts-disabled-description-' + i];
            this._currentAlertsDisabledContentsId.push(params['alerts-disabled-' + i]);
        }
        tooltip += this.getConfiguration()['alerts-disabled-end-description'];
    }
    
    return tooltip;
}


