/*
 *  Copyright 2012 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.editor.htmlexpert');

org.ametys.cms.editor.htmlexpert.ScriptEditor = function (config) 
{
	org.ametys.cms.editor.htmlexpert.ScriptEditor.superclass.constructor.call(this, config);
}

Ext.extend(org.ametys.cms.editor.htmlexpert.ScriptEditor, Ext.form.TextArea, {});

org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype._initialized;
org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype._codeMirror;
org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype._futureValue;

org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype.initComponent = function ()
{
	org.ametys.cms.editor.htmlexpert.ScriptEditor.superclass.initComponent.apply(this, arguments);
	
	this.addEvents('initialize');
	this.on({'resize': {fn: this._onResize, scope: this}});
	this.on({'move': {fn: this._onMove, scope: this}});
	this.on({'focus': {fn: this._onFocus, scope: this}});
	this.on({'initialize': {fn: this._init, scope: this}});
}

org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype.getCM = function()
{
    return this._codeMirror;
}

org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype._init = function ()
{
    if (this._futureValue != '')
    {
        this.setValue(_futureValue);
    }
}

org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype._onResize = function (editor, width, height)
{
	this._ensureInitialized();
	
	var el = Ext.select('.CodeMirror-scroll', true);
	if (el && height != null) 
	{
		el.setSize(width, height);
	}	
}

org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype._onMove = function(editor, x, y)
{
    this._ensureInitialized();
    
    var els = Ext.select('.CodeMirror', true);
    
    if (els && els.getCount() > 0)
    {
        var el = els.first();
        el.dom.style.position = 'relative';
        el.dom.style.left = x + 'px';
        el.dom.style.top = y + 'px';
    }
}

org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype._onFocus = function(editor, target, obj)
{
    if (this._codeMirror) 
    {
        this._codeMirror.focus();
    }
}

org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype._onCursorActivity = function ()
{
	// Styling the current cursor line
	if (this._hlLine != null)
	{
		this._codeMirror.setLineClass(this._hlLine, null);
	}
	this._hlLine = this._codeMirror.setLineClass(this._codeMirror.getCursor().line, "activeline");
}

org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype.onRender = function (ct, position)
{
	org.ametys.cms.editor.htmlexpert.ScriptEditor.superclass.onRender.call(this, ct, position);
}

org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype.getValue = function ()
{
	if (this._codeMirror) 
	{
		return this._codeMirror.getValue();
	}
	return this.initialConfig.value;
}

org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype.setValue = function (v)
{
    org.ametys.cms.editor.htmlexpert.ScriptEditor.superclass.setValue.call(this, v);
    
	if (this._codeMirror) 
	{
		this._codeMirror.setValue(v);
	}
	else
    {
	    this._futureValue = v;
    }
}

org.ametys.cms.editor.htmlexpert.ScriptEditor.prototype._ensureInitialized = function()
{
    var me = this;
    
    if (this._codeMirror == null)
    {
        this._codeMirror = CodeMirror.fromTextArea(document.getElementById(this.id), {
            
            mode: 'htmlmixed',
            value: me.initialConfig.value,  
            
            matchBrackets: true,
            lineNumbers: true,
            
            // Styling the current cursor line
            onCursorActivity: this._onCursorActivity.createDelegate(this),
        
            extraKeys: {
                "Ctrl-U": "undo",
                "Ctrl-Y": "redo"
            },
        
            initCallback: function() {
                me._initialized = true;
                me.fireEvent('initialize', true);
            }
        });
    }
}
