/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.ribbon.button');

/**
 * Create the control instance.
 * @param {String} id The unique identifier
 * @param {Map<id, value>} configuration The configuration of the control instance.
 * @param {String} pluginName The name of the declaring plugin
 * @class This class implements the behavior for an action of the editor. The 'context' configuration allow to enable/disable the button.
 */
org.ametys.cms.ribbon.button.EditorButton = function(id, configuration, pluginName)
{
	org.ametys.cms.ribbon.button.EditorButton.superclass.constructor.call(this, id, configuration, pluginName);

	eval("this._selectionListener = " + this.getConfiguration()['selection-listener'] + ";");
}

Ext.extend(org.ametys.cms.ribbon.button.EditorButton, org.ametys.ribbon.control.button.ActionButton, {});

/**
 * Get the messages to determine enable / disable
 * @param {org.ametys.messagebus.message.Message[]} messages The messages received
 */
org.ametys.cms.ribbon.button.EditorButton.prototype.onMessage = function(messages)
{
	for (var i = 0; i < messages.length; i++)
	{
		var message = messages[i];
		if (message.getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = message.getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, function(target) { return target.getType() == "node";});

			var node = target != null ? target.getParameters()['object'] : null;

			if (this._selectionListener)
			{
				this._selectionListener(this, target, node);
			}
			else
			{
				this.getElements().each(function(element) {element.setDisabled(target == null);});
			}
		}
	}
}
