/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.ribbon.button');

org.ametys.cms.ribbon.button.Menu = function(id, configuration, pluginName) 
{
	org.ametys.cms.ribbon.button.Menu.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.cms.ribbon.button.Menu, org.ametys.ribbon.control.button.ActionButton, {});

/**
 * @private
 * @property _lastParams The params received on the last refresh
 */
org.ametys.cms.ribbon.button.Menu.prototype._lastParams;

org.ametys.cms.ribbon.button.Menu.prototype.createControl = function(size, colspan)
{
	var element = org.ametys.cms.ribbon.button.Menu.superclass.createControl.call(this, size, colspan);
	element._menuItems = {};
	element._galleryItems = {};
	
	var items = [];
	
	var galleryCount = parseInt(this.getConfiguration()["gallery-size"]); 
	if (galleryCount > 0)
	{
		element._galleryGroup = {};
		var gallery = new org.ametys.fluent.menu.GalleryItem ({
	    	panelConfig: 
	 		{
	    		items : [],
	    		width: 360
	 		}
	    });
		
		var buttons = [];
		for (var i=0;  i < galleryCount; i++)
		{
			var group = this.getConfiguration()["gallery-" + i + "-group"];
			if (!element._galleryGroup[group])
			{
				element._galleryGroup[group] = new Ext.Panel({title: group});
				gallery.panelConfig.items.push(element._galleryGroup[group]);
			}
			
			var actionId = this.getConfiguration()["gallery-" + i];
			element._galleryItems["gallery-" + i] = new org.ametys.fluent.menu.GalleryButton ({
				actionId: actionId,
				groupEl: element._galleryGroup[group],
				
				text: this.getConfiguration()["gallery-" + i + "-label"],
				tooltip: this.getConfiguration()["gallery-" + i + "-description"],
				iconSmall: context.contextPath + this.getConfiguration()["gallery-" + i + "-iconSmall"], 
				iconLarge: context.contextPath + this.getConfiguration()["gallery-" + i + "-iconMedium"], 
				handler: function (button) {eval(this.ribbonControl.getConfiguration()["gallery-" + this.index + "-action"] + "(button, this.pressed, this.ribbonControl.getConfiguration(), this.index)")},
				
				size: 'large',
	 		 	enableToggle: this.getConfiguration()["gallery-" + i + "-toggle-group"] != null,
 		 		allowDepress: false,
	 		 	toggleGroup: this.getConfiguration()["gallery-" + i + "-toggle-group"],
	 		 	
				ribbonControl: this,
				index: i
			}); 
			element._galleryGroup[group].add(element._galleryItems["gallery-" + i]);
		}
		items.push(gallery);
	}
	
	var menuCount = parseInt(this.getConfiguration()["menu-size"]); 
	if (menuCount > 0)
	{
		for (var i=0;  i < menuCount; i++)
		{
			var actionId = this.getConfiguration()["menu-" + i];
			element._menuItems["menu-" + i] = new Ext.menu.Item ({
				actionId: actionId,
				
				text: this.getConfiguration()["menu-" + i + "-label"],
				tooltip: this.getConfiguration()["menu-" + i + "-description"],
				icon: context.contextPath + this.getConfiguration()["menu-" + i + "-iconSmall"], 
				handler: function (button) {eval(this.ribbonControl.getConfiguration()["menu-" + this.index + "-action"] + "(button, this.ribbonControl.getConfiguration(), this.index)")},
				
				ribbonControl: this,
				index: i
			}); 
			items.push(element._menuItems["menu-" + i]);
		}
	}
	
	element.on('menushow', this._onMenuShow, this);
	element.menu = new Ext.menu.Menu({
		items: items
	})
	
	if (this._configuration['action'] != null || element.enableToggle)
	{
		element.split = true;
	}

	return element;
}

/**
 * @private
 * This handler is called when the toogle button state change
 */
org.ametys.cms.ribbon.button.Menu.prototype._onMenuShow = function(button, event)
{
	for (var itemId in button._menuItems)
	{
		var item = button._menuItems[itemId];
		if (this.getConfiguration()[itemId + '-display-listener'])
		{
			eval(this.getConfiguration()[itemId + '-display-listener'] + "(item);");
		}
	}

	for (var itemId in button._galleryItems)
	{
		var item = button._galleryItems[itemId];
		if (this.getConfiguration()[itemId + '-display-listener'])
		{
			eval(this.getConfiguration()[itemId + '-display-listener'] + "(item);");
		}
	}
}

/**
 * Open a tool from a button.
 */
org.ametys.cms.ribbon.button.OpenTool = function(id, configuration, pluginName)
{
    var role = configuration['role'];
                                                     
    org.ametys.uitool.ToolsManager.getInstance().openTool(role, {});
}

/**
 * Open a tool from a menu item.
 */
org.ametys.cms.ribbon.button.MenuOpenTool = function(button, configuration, index)
{
    var role = configuration['menu-' + index + '-role'];
    
    org.ametys.uitool.ToolsManager.getInstance().openTool(role, {});
}

