/*
 *  Copyright 2012 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.ribbon.button');

org.ametys.cms.ribbon.button.UniqueInstanceToolButtonWithMenu = function(id, configuration, pluginName) 
{
	org.ametys.cms.ribbon.button.UniqueInstanceToolButtonWithMenu.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.cms.ribbon.button.UniqueInstanceToolButtonWithMenu, org.ametys.cms.ribbon.button.ToggleButtonWithMenu, {});

org.ametys.cms.ribbon.button.UniqueInstanceToolButtonWithMenu.prototype.onToggle = function(button, state)
{
	var control = org.ametys.ribbon.RibbonManager.getInstance().getControls()[this.controlId];
	if (state)
	{
		org.ametys.uitool.ToolsManager.getInstance().openTool(control.getConfiguration()['role'], control.getOpenToolParameters());
	}
	else
	{
		var tool = org.ametys.uitool.ToolsManager.getInstance().getTool(control.getConfiguration()['role']);
		
		if (tool.toolsPanel.getActiveTab() != tool.parent)
		{
			org.ametys.uitool.ToolsManager.getInstance().activateTool (tool, tool._factory._config['default-location']);
			this.toggle(true);
		}
		else
		{
			tool.close();
		}
	}
}

/**
 * Get the parameters needed by the factory to open the tool
 * @return {Map} params The parameters needed by the factory to open the tool. Can not be null.
 */
org.ametys.cms.ribbon.button.UniqueInstanceToolButtonWithMenu.prototype.getOpenToolParameters = function()
{
	return {};
}

org.ametys.cms.ribbon.button.UniqueInstanceToolButtonWithMenu.prototype.onMessage = function (messages)
{
	var shouldBeState = null;
	
	for (var i = 0; i < messages.length; i++)
	{
		var message = messages[i];
		
		switch (message.getType().getName())
		{
			case org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLOPENED:
				if (this._isInTarget(message.getTargets()))
				{
					shouldBeState = true;
				}
				break;
			case org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLCLOSED:
				if (this._isInTarget(message.getTargets()))
				{
					shouldBeState = false;
				}
				break;
		}
	}
	
	if (shouldBeState != null)
	{
		this.getElements().each(function(element) { element.toggle(shouldBeState, true); })
	}
}

/**
 * @private
 * This method returns true if current tool id is one of the target
 */
org.ametys.cms.ribbon.button.UniqueInstanceToolButtonWithMenu.prototype._isInTarget = function(targets)
{
	for (var i = 0; i < targets.length; i++)
	{
		var target = targets[i];
		
		if (target.getType() == "tool" && target.getParameters()['id'] == this.getConfiguration()['role'])
		{
			return true;
		}
		
		if (this._isInTarget(target.getSubTargets()))
		{
			return true;
		}
	}
	
	return false;
}