/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.ribbon.tab');

/**
 * Do not call this contructor
 * @constructor
 * @class This class handles the tab that shows when a tool has the focus
 * @param {String} id The unique tab identifier
 * @param {Map<id, value>} configuration The configuration of the control instance. The key 'tool-id' represents the unique identifier of the tool waited. The key 'factory-id' represents the identifier of the factory of the tool waited.
 * @param {String} The name of the plugin where the control was declared
 * @param {Ext.Panel} The panel of the tab
 */
org.ametys.cms.ribbon.tab.ToolTab = function(id, configuration, pluginName, tabPanel) 
{
	org.ametys.cms.ribbon.tab.ToolTab.superclass.constructor.call(this, id, configuration, pluginName, tabPanel);
	
	this._toolRegExp = new RegExp(configuration['factory-id']);
};

Ext.extend(org.ametys.cms.ribbon.tab.ToolTab, org.ametys.ribbon.control.RibbonTab, {});


/**
 * @private
 * @property {RegExp} _toolRegExp The regexp for the tool lookup
 */
org.ametys.cms.ribbon.tab.ToolTab.prototype._toolRegExp;

/**
 * Activate or deactivate the tab depending on the selection
 * @param {org.ametys.messagebus.message.Message[]} messages The messages received.
 * @private
 */
org.ametys.cms.ribbon.tab.ToolTab.prototype.onMessage = function (messages)
{
	for (var i=messages.length-1; i >= 0; i--)
	{
		var targets = messages[i].getTargets();
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLFOCUSES)
		{
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, this._testTarget.createDelegate(this));
			
			if (target == null)
			{
				this.hide();
			}
			else
			{
				this.show(messages[i].getType().getParameters() != null && messages[i].getType().getParameters()["creation"] == true);
			}
		}
		else if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLFOCUSES
			|| messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLBLUR)
		{
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, this._testTarget.createDelegate(this));
			if (target != null)
			{
				// hides tab if visible
				this.hide();
			}
		}
	}
}

/**
 * @private
 * Test a target. Does it match ?
 * @return {org.ametys.messagebus.message.MessageTarget} The target to check
 */
org.ametys.cms.ribbon.tab.ToolTab.prototype._testTarget = function(target)
{
	return target.getType() == "tool" && this._toolRegExp.test(target.getParameters()['id']);
}
