/*
 *  Copyright 2012 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.ribbon.tab');

org.ametys.cms.ribbon.tab.WorkflowContentTab = function(id, configuration, pluginName, tabPanel) 
{
	org.ametys.cms.ribbon.tab.WorkflowContentTab.superclass.constructor.call(this, id, configuration, pluginName, tabPanel);
	
	if (configuration['content-type'] != null)
	{
		this._cTypeRegExp = new RegExp(configuration['content-type']);
	}
	
	if (configuration['workflow-name'] != null)
	{
	    this._workflowNameRegexp = new RegExp(configuration['workflow-name']);
	}
}

Ext.extend(org.ametys.cms.ribbon.tab.WorkflowContentTab, org.ametys.cms.ribbon.tab.SelectionTab, {});

/**
 * @private
 * @property {RegExp} _cTypeRegExp The content type of the target lookup
 */
org.ametys.cms.ribbon.tab.WorkflowContentTab.prototype._cTypeRegExp;

/**
 * @private
 * @property {RegExp} _cTypeRegExp The content type of the target lookup
 */
org.ametys.cms.ribbon.tab.WorkflowContentTab.prototype._workflowNameRegexp;

/**
 * @private
 * Test a target. Does it match ?
 * @return {org.ametys.messagebus.message.MessageTarget} The target to check
 */
org.ametys.cms.ribbon.tab.WorkflowContentTab.prototype._testTarget = function(target)
{
	var matchSelection = org.ametys.cms.ribbon.tab.WorkflowContentTab.superclass._testTarget.call(this, target);
	if (!matchSelection)
	{
		return false;
	}
	
    if (this._workflowNameRegexp != null && !this._workflowNameRegexp.test(target.getParameters().workflowName))
    {
        return false;
    }
    
	var cType = target.getParameters().type;
	if (this._cTypeRegExp != null && !this._cTypeRegExp.test(cType))
	{
	    return false;
	}
	
	return true;
}
