/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.tool.profile');

org.ametys.cms.tool.profile.ProfileContextAndUGButton = function(id, configuration, pluginName) 
{
	org.ametys.cms.tool.profile.ProfileContextAndUGButton.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.cms.tool.profile.ProfileContextAndUGButton, org.ametys.ribbon.control.button.ActionButton, {});

org.ametys.cms.tool.profile.ProfileContextAndUGButton.prototype.onMessage = function (messages)
{
	// SELECTION
	for (var i=messages.length - 1; i >= 0; i--)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, this._testTarget.createDelegate(this));
			
			// No user or group selected
			if (target == null)
			{
				this.getElements().each(function(element) { element.disable(); });
			}
			else
			{
				this.getElements().each(function(element) { element.refreshing(); element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["refresh-description"]);
				
				var params = {};
				params.profile = target.getParameters()['id'];
				params.context = target.getParameters()['context'];
				org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, params);
			}
			
			return;
		}
	}
}

org.ametys.cms.tool.profile.ProfileContextAndUGButton.prototype.configurationError = function(response)
{
	this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(true); });
	this.updateTooltip(this.getConfiguration()["default-description"]);
}

org.ametys.cms.tool.profile.ProfileContextAndUGButton.prototype.configurationChanged = function(params)
{
	this.getElements().each(function(element) { element.stopRefreshing(); });
	
	if (params['can-delegate-rights-on-profile'] != null)
	{
		this.getElements().each(function(element) { element.enable(); });
	}
	else
	{
		this.updateTooltip(this.getConfiguration()["no-right-description"]);
		this.getElements().each(function(element) { element.disable(); });
	}
	
	this._lastParams = params;
}

org.ametys.cms.tool.profile.ProfileContextAndUGButton.prototype._testTarget = function (target)
{
	if (target.getType() == 'profile' && target.getParameters().context)
	{
		var subTarget = org.ametys.messagebus.message.MessageTargetHelper.findFirst(target.getSubTargets(), this._testUGTarget.createDelegate(this))
		return subTarget != null;
	}
	
	return false;
}

org.ametys.cms.tool.profile.ProfileContextAndUGButton.prototype._testUGTarget = function (target)
{
	return target.getType() == "user" || target.getType() == "group";
}




