/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.tool.profile');

org.ametys.cms.tool.profile.ProfileContextButton = function(id, configuration, pluginName) 
{
	org.ametys.cms.tool.profile.ProfileContextButton.superclass.constructor.call(this, id, configuration, pluginName);
};

Ext.extend(org.ametys.cms.tool.profile.ProfileContextButton, org.ametys.ribbon.control.button.ActionButton, {});

/**
 * @private
 * @property _lastParams The params received on the last refresh
 */
org.ametys.cms.tool.profile.ProfileContextButton.prototype._lastParams;

/**
 * @private
 * @property {String[]} _currentGroupsId The groups' id currently selected or null. 
 */
org.ametys.cms.tool.profile.ProfileContextButton.prototype._currentGroupsId;

org.ametys.cms.tool.profile.ProfileContextButton.prototype.onMessage = function (messages)
{
	// SELECTION
	for (var i=messages.length - 1; i >= 0; i--)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			
			var allTargets = org.ametys.messagebus.message.MessageTargetHelper.findAll(targets, this.checkTarget());
			
			// No profile selected
			if (allTargets.length == 0)
			{
				this.getElements().each(function(element) { element.disable(); });
			}
			else
			{
				this.getElements().each(function(element) { element.refreshing(); element.setDisabled(true); });
				this.updateTooltip(this.getConfiguration()["refresh-description"]);
				
				var params = {};
				params.profile = allTargets[0].getParameters()['id'];
				params.context = allTargets[0].getParameters()['context'];
				org.ametys.ribbon.RibbonManager.getInstance().refreshElement('control', this, params);
			}
			return;
		}
	}
}

org.ametys.cms.tool.profile.ProfileContextButton.prototype.configurationError = function(response)
{
	this.getElements().each(function(element) { element.stopRefreshing(); element.setDisabled(true); });
	this.updateTooltip(this.getConfiguration()["default-description"]);
}

org.ametys.cms.tool.profile.ProfileContextButton.prototype.configurationChanged = function(params)
{
	this.getElements().each(function(element) { element.stopRefreshing(); });
	
	if (params['can-delegate-rights-on-profile'] != null)
	{
		this.updateTooltip(this.getConfiguration()["default-description"]);
		this.getElements().each(function(element) { element.enable(); });
	}
	else
	{
		this.updateTooltip(this.getConfiguration()["no-right-description"]);
		this.getElements().each(function(element) { element.disable(); });
	}
	
	this._lastParams = params;
}

org.ametys.cms.tool.profile.ProfileContextButton.prototype._convertToId = function (targets)
{
	var targetsIds = [];
	for (var i = 0; i < targets.length; i++)
	{
		targetsIds.push(targets[i].getParameters()['id']);
	}
	return targetsIds;
}

org.ametys.cms.tool.profile.ProfileContextButton.prototype.checkTarget = function ()
{
	return function (target) { return target.getType() == 'profile' && target.getParameters().context; };
}
