/*
 *  Copyright 2011 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.tool.profile');

org.ametys.cms.tool.profile.ProfilesToolStateButton = function(id, configuration, pluginName)
{
	org.ametys.cms.tool.profile.ProfilesToolStateButton.superclass.constructor.call(this, id, configuration, pluginName);
}

Ext.extend(org.ametys.cms.tool.profile.ProfilesToolStateButton, org.ametys.ribbon.control.button.ActionButton, {});

org.ametys.cms.tool.profile.ProfilesToolStateButton.prototype.onMessage = function (messages)
{
	for (var i=messages.length-1; i >= 0; i--)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLFOCUSES)
		{
			var targets = messages[i].getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, function(target) { return target.getType() == "tool"});
			var currentToolId = target.getParameters()['id'];
				
			if (this.getConfiguration()['tool-id'] != null && currentToolId == this.getConfiguration()['tool-id'])
			{
				this._setDisabledOrToggleState ();
				return;
			}
			
			this.getElements().each(function(element) { element.toggle(false); })
		}
		
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLFOCUSES || messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLBLUR)
		{
			this.getElements().each(function (element) { element.disable(); });
		}
		
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLSTATECHANGED)
		{
			var targets = messages[i].getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, function(target) { return target.getType() == "tool"});
				
			if (this.getConfiguration()['tool-id'] != null && target.getParameters()['id'] == this.getConfiguration()['tool-id'])
			{
				this._setDisabledOrToggleState ();
				return;
			}
			
			this.getElements().each(function(element) { element.toggle(false); })
		}
		
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, function(target) { return target.getType() == "profile-right"});
			
			if (target == null)
			{
				this._hasSelection = false;
				
				var tooltip = this.getConfiguration()['default-description'];
				tooltip += '<br/><br/>';
				tooltip += this.getConfiguration()['no-selection'];
				this.updateTooltip (tooltip);
				
				return;
			}
			else
			{
				this._hasSelection = true;
				var disabled = this._disabledState || !this._hasSelection;
				this.getElements().each(function(element) { element.setDisabled(disabled); });
				
				this.updateTooltip (this.getConfiguration()['default-description']);
			}
		}
	}
}

org.ametys.cms.tool.profile.ProfilesToolStateButton.prototype._setDisabledOrToggleState = function ()
{
	var tool = org.ametys.uitool.ToolsManager.getInstance().getTool(this.getConfiguration()['tool-id']);
	if (tool != null)
	{
		var params = {};
		if (this.getConfiguration()['state'])
		{
			params.mode = this.getConfiguration()['state'];
		}
		
		var toggleOrEnable = tool.queryCommandState (this.getConfiguration()['command-state'], params);
		
		this._disabledState = false;
		
		if (this.getConfiguration()['btn-state-mode'] == 'toggle')
		{
			this.getElements().each(function(element) { element.toggle(toggleOrEnable); })
		}
		else
		{
			this._disabledState = !toggleOrEnable;
		}
		
		var disabled = this._disabledState || !this._hasSelection;
		this.getElements().each(function(element) { element.setDisabled(disabled); })
	}
}