/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

// Ametys Namespace
Ext.namespace('org.ametys.fluent.menu');

/**
 * Creates a gallery item for a menu
 * 
 * @class This class is the item to put in menu to make a gallery
 * @extends Ext.menu.BaseItem
 * @constructor
 * @param {Object} config Configuration options
 */
org.ametys.fluent.menu.GalleryItem = function(config) 
{
	if (config.panelConfig == null)
	{
		config.panelConfig = {};
	}
	Ext.applyIf
	(
		config.panelConfig,
		{
			bodyCssClass: 'x-fluent-gallery-wrapper',
			bwrapCssClass: 'x-fluent-gallery-bwrapper',
			
			listeners: { 'render': this._onPanelRender },
			
			width: 288
		}
	);

	org.ametys.fluent.menu.GalleryItem.superclass.constructor.call(this, config);

	this.on('render', this._onRender, this);
}

Ext.extend
(
	org.ametys.fluent.menu.GalleryItem, 
	Ext.menu.BaseItem, 
	{
		ctype: 'org.ametys.fluent.menu.GalleryItem'
	}
);

/**
 * @property {Ext.Panel} _galleryPanel The gallery panel. Initialized during render
 * @private
 */
org.ametys.fluent.menu.GalleryItem.prototype._galleryPanel;

/**
 * @private
 * Listener on render to draw the gallery
 */
org.ametys.fluent.menu.GalleryItem.prototype._onRender = function()
{
	this.panelConfig.renderTo = this.container;

	this._galleryPanel = new Ext.Panel(this.panelConfig);
}

org.ametys.fluent.menu.GalleryItem.prototype._onPanelRender = function()
{
	this.items.each
	(
		function(item) 
		{
			if (item.doLayout)
			{
				item.on('afterrender', function(c) {c.doLayout(false, true)}, this);
			}
		}
	);
}
