/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.messagebus.bus');

/**
 * Do not call this constructor. Use getInstance() instead.
 * @constructor
 * @class This class has a single instance. Use getInstance() to get it.
 * This class allow to create a message. See createMessage for more information.
 */
org.ametys.messagebus.bus.MessageBuilder = function()
{
	this._targetFactories = {};
}

/**
 * @property {org.ametys.messagebus.bus.MessageBuilder} _instance Unique instance.
 * @private
 */
org.ametys.messagebus.bus.MessageBuilder._instance;
/**
 * This method returns the unique instance of the org.ametys.messagebus.bus.MessageBuilder
 * @static
 * @returns {org.ametys.messagebus.bus.MessageBuilder} The unique instance of the org.ametys.messagebus.bus.MessageBuilder class.
 */
org.ametys.messagebus.bus.MessageBuilder.getInstance = function()
{
	if (org.ametys.messagebus.bus.MessageBuilder._instance == null)
	{
		org.ametys.messagebus.bus.MessageBuilder._instance = new org.ametys.messagebus.bus.MessageBuilder();
	}
	return org.ametys.messagebus.bus.MessageBuilder._instance;
}

/**
 * @property {Map<type, factory>} _targetFactories This map keep the factories associated with their id
 * @private
 */
org.ametys.messagebus.bus.MessageBuilder.prototype._targetFactories;

/**
 * Adds a new factory to the list of declared factories.
 * @param {String} type The type of target that will be concerned by that factory
 * @param {org.ametys.messagebus.bus.MessageTargetFactory} factory The instance of the factory that will be used to create message of that kind.
 */
org.ametys.messagebus.bus.MessageBuilder.prototype.addTargetFactory = function(type, factory)
{
	this._targetFactories[type] = factory;
}

/**
 * Creates an event. Use the matching factory to do so.
 * @param {String} type See {@link org.ametys.messagebus.message.MessageType}.
 * @param {Map<id, value>} typeParams See {@link org.ametys.messagebus.message.MessageType}
 * @param {org.ametys.messagebus.message.MessageTarget[]} targets The targets of the message. Can not be null but may be empty
 * @return {org.ametys.messagebus.message.Message} The created message. Can not be null.
 * @throws an exception if no factory matches.
 */
org.ametys.messagebus.bus.MessageBuilder.prototype.createMessage = function(type, typeParams, targets)
{
	var messageType = new org.ametys.messagebus.message.MessageType(type, typeParams);
	
	return new org.ametys.messagebus.message.Message(messageType, targets);
}

/**
 * Creates a target. Use the matching factory to do so. See {@link org.ametys.messagebus.message.MessageTarget}.
 * @param {String} target See {@link org.ametys.messagebus.message.MessageTarget}.
 * @param {Map<id, value>} targetParams See {@link org.ametys.messagebus.message.MessageTarget}.
 * @return {org.ametys.messagebus.message.MessageTarget} The created target. Can not be null.
 * @throws an exception if no factory matches.
 */
org.ametys.messagebus.bus.MessageBuilder.prototype.createTarget = function(target, targetParams)
{
	var factory = this._targetFactories[target];
	if (factory == null)
	{
		factory = this._targetFactories["*"];
	}
	if (factory == null)
	{
		throw "Can not create an event because target '" + target + "' is unknow by the org.ametys.messagebus.bus.MessageBuilder";
	}
	
	var messageTarget = factory.createTarget(target, targetParams);
	
	return messageTarget;
}


/**
 * Creates a Array of target. Use the matching factory to do so. See {@link org.ametys.messagebus.message.MessageTarget}.
 * @param {String} target See {@link org.ametys.messagebus.message.MessageTarget}.
 * @param {Map<id, value>} targetParams See {@link org.ametys.messagebus.message.MessageTarget}.
 * @return {org.ametys.messagebus.message.MessageTarget} The created target. Can not be null.
 * @throws an exception if no factory matches.
 */
org.ametys.messagebus.bus.MessageBuilder.prototype.createTargets = function(target, targetParams)
{
	var factory = this._targetFactories[target];
	if (factory == null)
	{
		factory = this._targetFactories["*"];
	}
	if (factory == null)
	{
		throw "Can not create an event because target '" + target + "' is unknow by the org.ametys.messagebus.bus.MessageBuilder";
	}
	
	var messageTargets = factory.createTargets(target, targetParams);
	
	return messageTargets;
}
