/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.messagebus.message');

/**
 * Creates a message.
 * @constructor
 * @class The class represents a message for the bus. A message is composed of a type and a target (such as 'page X is created by user Y').
 * @param {org.ametys.messagebus.message.MessageType} type The type of message. Can not be null.
 * @param {org.ametys.messagebus.message.MessageTarget[]} targets The targets of message. Can not be null but can be empty.
 */
org.ametys.messagebus.message.Message = function (type, targets)
{
	this._type = type;
	this._targets = targets;
}

/**
 * @private
 * @property {org.ametys.messagebus.message.MessageType} _type The type of the message
 */
org.ametys.messagebus.message.Message.prototype._type;
/**
 * Get the type of the event.
 * @return {org.ametys.messagebus.message.MessageType} The type of the event. Can not be null.
 */
org.ametys.messagebus.message.Message.prototype.getType = function()
{
	return this._type;
}

/**
 * @private
 * @property {org.ametys.messagebus.message.MessageTarget[]} _targets The target of the message
 */
org.ametys.messagebus.message.Message.prototype._targets;
/**
 * Get the target of the event.
 * @return {org.ametys.messagebus.message.MessageTarget[]} The targets of the event. Can not be null.
 */
org.ametys.messagebus.message.Message.prototype.getTargets = function()
{
	return this._targets;
}
