/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.messagebus.message');

/**
 * @constructor
 * @private
 * @class This class is an helper on message target
 */
org.ametys.messagebus.message.MessageTargetHelper = function()
{
	// empty
}

/**
 * Find in the targets and their subtargets, the first target matching a condition.
 * @param {org.ametys.messagebus.message.MessageTarget[]} targets An array of targets. Cannot be null but empty 
 * @param {function} testFunction The function that will test if a target match. <code>function testMatch(target) { return target.getType() == "content"; }</code>
 * @return {org.ametys.messagebus.message.MessageTarget[]} The targets corresponding. Can be null.
 */
org.ametys.messagebus.message.MessageTargetHelper.findFirst = function(targets, testFunction)
{
	for (var i=0; i < targets.length; i++)
	{
		if (testFunction(targets[i]))
		{
			return targets[i];
		}
		
		var subSearch = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets[i].getSubTargets(), testFunction);
		if (subSearch != null)
		{
			return subSearch;
		}
	}
	return null;
}

/**
 * Find in the targets and their subtargets, the targets matching a condition.
 * @param {org.ametys.messagebus.message.MessageTarget[]} targets An array of targets. Cannot be null but empty 
 * @param {function} testFunction The function that will test if a target match. <code>function testMatch(target) { return target.getType() == "content"; }</code>
 * @return {org.ametys.messagebus.message.MessageTarget[]} The targets corresponding. Cannot be null.
 */
org.ametys.messagebus.message.MessageTargetHelper.findAll = function(targets, testFunction)
{
	var allTargets = [];
	
	for (var i=0; i < targets.length; i++)
	{
		if (testFunction(targets[i]))
		{
			allTargets.push(targets[i]);
		}
		
		var subSearch = org.ametys.messagebus.message.MessageTargetHelper.findAll(targets[i].getSubTargets(), testFunction);
		allTargets = allTargets.concat(subSearch);
	}
	
	return allTargets;
}
