/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.messagebus.message');

/**
 * Creates a message type.
 * MessageType are used at during the creation of a {@link org.ametys.messagebus.message.Message}
 * @constructor
 * @class This class represents the type of a {@link org.ametys.messagebus.message.Message}. Such as "created by user X".
 * @param {String} name The name of the type of event (such as "selectionChanged" or "created"). Must not be null.
 * @param {Map<id, value>} params A map of parameters that allow to determine the target. Values depends on the target type. Target of type 'content' will need a identifier in the params. Must not be null.
 */
org.ametys.messagebus.message.MessageType = function (name, parameters)
{
	this._name = name;
	this._parameters = parameters;
}

/**
 * @private
 * @property {String} _name See associated parameter in constructor for details
 */
org.ametys.messagebus.message.MessageType.prototype._name;
/**
 * See the associated parameter in the constructor for more details.
 * @return {Map<id, value>} The name of the type. Can not be null.
 */
org.ametys.messagebus.message.MessageType.prototype.getName = function ()
{
	return this._name;
}

/**
 * @private
 * @property {Map<id, value>} _parameters See associated parameter in constructor for details
 */
org.ametys.messagebus.message.MessageType.prototype._parameters;
/**
 * See the associated parameter in the constructor for more details.
 * @return {Map<id, value>} The parameters of the target. Can not be null but empty.
 */
org.ametys.messagebus.message.MessageType.prototype.getParameters = function ()
{
	return this._parameters;
}
