/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.cms.control');

org.ametys.cms.control.InputField = function(id, configuration, pluginName)
{
	org.ametys.cms.control.InputField.superclass.constructor.call(this, id, configuration, pluginName);
	
	eval("this._selectionListener = " + this.getConfiguration()['selection-listener'] + ";");
}

Ext.extend(org.ametys.cms.control.InputField, org.ametys.ribbon.control.RibbonControl, {});

org.ametys.cms.control.InputField.prototype.createControl = function(size, colspan)
{
	var initFn = this.getConfiguration()['initialize'];
	if (initFn != null)
	{
		eval(initFn + "(this.getId(), this.getConfiguration(), this.getPluginName());");
	}
	
	var listeners = {};
	
	if (this.getConfiguration()['default-description'] != "")
	{
		var conf = this.getConfiguration(); 
	
		listeners['render'] = function()
					          {
									var tooltip = this.tooltip || org.ametys.fluent.Tooltip.create (
																									conf['label'],
																									conf['icon-large'] || conf['icon-medium'] || conf['icon-small'] || conf['icon'],
																									conf['default-description'], 
																									conf['footer']
																								);
						
									if(Ext.isObject(tooltip))
							        {
							            Ext.QuickTips.register(Ext.apply({
							                  target: this.el.dom.id
							            }, tooltip));
							            if (this.label)
							            {
								            Ext.QuickTips.register(Ext.apply({
								                  target: this.label.id
								            }, tooltip));
							            }
							        }
							        else
							        {
							            this.el.dom['qtip'] = tooltip;
							            if (this.label)
							            {
							            	this.label.dom['qtip'] = tooltip;
							            }
							        }
					          }
	}
	
	for (var key in this.getConfiguration())
	{
		if (/^on(.*)$/.test(key))
		{
			listeners[RegExp.$1] = eval(this.getConfiguration()[key]);
		}
	}
	
	var width = this._getWidth(size);
	var elementConf = this._getConf(width, listeners);
	
	var panel = new Ext.Panel ({
	       layout: "form",
	       defaultType : 'textfield',
	       labelAlign: size == 'large' && elementConf.fieldLabel ? 'top' : 'left',
	       labelWidth : this.getConfiguration()['label-width'] != null ? Number(this.getConfiguration()['label-width']) : 60,
	       colspan: colspan,
	    	
	       items:[elementConf]
	});

	var element = panel.items.get(0);
	
	this.getElements().add(element);
	
	return panel;
}

org.ametys.cms.control.InputField.prototype._getWidth = function(size)
{
	var width = this.getConfiguration()['width'] != null ? Number(this.getConfiguration()['width']) : 'auto';
	width = this.getConfiguration()['width-' + size] != null ? Number(this.getConfiguration()['width-' + size]) : width;
	return width;
}
org.ametys.cms.control.InputField.prototype._getConf = function(width, listeners)
{
	return {
		cls: this.getConfiguration()['cls'],
		readOnly: this.getConfiguration()['readOnly'] == "true",
			
		inputType: this.getConfiguration()['input-type'] != null ? this.getConfiguration()['input-type'] : 'text', 
    	xtype: this.getConfiguration()['input-xtype'] != null ? this.getConfiguration()['input-xtype'] : 'textfield', 

    	hideLabel: this.getConfiguration()['label'] == null,
    	allowNegative: this.getConfiguration()['allowNegative'] != null ? this.getConfiguration()['allowNegative'] == "true" : false,
    	allowDecimals: this.getConfiguration()['allowDecimals'] != null ? this.getConfiguration()['allowDecimals'] == "true" : false,
    	fieldLabel: this.getConfiguration()['label'],
    	name: this.getConfiguration()['name'],
    	height: this.getConfiguration()['height'],
    	emptyText : this.getConfiguration()['empty-text'],
    	
    	enableKeyEvents : true,
    	
    	width: width,
    	disabled: this.getConfiguration()['disabled'] == "true",
    	
    	listeners: listeners
    };
}

/**
 * Get the messages to determine enable / disable
 * @param {org.ametys.messagebus.message.Message[]} messages The messages received
 */
org.ametys.cms.control.InputField.prototype.onMessage = function(messages)
{
	for (var i = 0; i < messages.length; i++)
	{
		var message = messages[i];
		if (message.getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = message.getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, function(target) { return target.getType() == "node";});

			var node = target != null ? target.getParameters()['object'] : null;

			if (this._selectionListener)
			{
				this._selectionListener(this, target, node);
			}
			else
			{
				this.getElements().each(function(element) { element.setDisabled(target == null); });
			}
		}
	}
}

org.ametys.cms.control.InputField.prototype.onRender = function(ct, position)
{
	org.ametys.cms.control.InputField.superclass.onRender.call(this, ct, position);

	if (this.tooltip)
	{
		this.setTooltip(tooltip, true);
	}
}
