/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.ribbon.control');

/**
 * Creates the control. There is one instance per control. Extends this method to initialize your component.
 * @param {String} id The unique tab identifier
 * @param {Map<id, value>} configuration The configuration of the instance.
 * @param {String} The name of the plugin where the tab was declared
 * @param {Ext.Panel} The panel of the tab
 * @class This interface is used for ribbon control. You must implement it to creates a control.
 * @constructor
 */
org.ametys.ribbon.control.RibbonTab = function(id, configuration, pluginName, tabPanel)
{
	org.ametys.ribbon.control.RibbonTab.superclass.constructor.call(this, id, configuration, pluginName);
	
	this._tabPanel = tabPanel;
	this._wasActive = true;
	this._alreadyActivated = false;
	
	org.ametys.messagebus.MessageBus.getInstance().register(this);
}

/**
 * @private
 * @property {Ext.Panel} _tabPanel The extjs tab panel
 */
org.ametys.ribbon.control.RibbonTab.prototype._tabPanel;

Ext.extend(org.ametys.ribbon.control.RibbonTab, org.ametys.ribbon.control.RibbonElement, {});

/**
 * Show the tab item
 * @param {boolean} activate Set to true to also activate the tab
 */
org.ametys.ribbon.control.RibbonTab.prototype.show = function(activate)
{
	this._tabPanel.showContextualTab(activate);
}

/**
 * Hide the tab item
 */
org.ametys.ribbon.control.RibbonTab.prototype.hide = function()
{
	this._tabPanel.hideContextualTab();
}
