/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.ribbon.control.button');

/**
 * ToolActionButton that moreover is disabled when a page is not selected
 * The configuration of the control instance. The key 'target-type' represents the name of the target that will make the button enable when this kind of target is in the {@see org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED} message. You can optionnaly set a subtarget-type that do the same on the immediate first child
 */
org.ametys.ribbon.control.button.ToolActionSelectionButton = function(id, configuration, pluginName)
{
	org.ametys.ribbon.control.button.ToolActionSelectionButton.superclass.constructor.call(this, id, configuration, pluginName);

	this._targetRegExp = new RegExp(configuration['target-type']);
	if (configuration['subtarget-type'] != null)
	{
		this._subtargetRegExp = new RegExp(configuration['subtarget-type']);
	}
}

Ext.extend(org.ametys.ribbon.control.button.ToolActionSelectionButton, org.ametys.ribbon.control.button.ToolActionButton, {});

/**
 * @private
 * @property {RegExp} _targetRegExp The regexp for the target lookup
 */
org.ametys.ribbon.control.button.ToolActionSelectionButton.prototype._targetRegExp;

/**
 * @private
 * @property {RegExp} _subtargetRegExp The regexp for the sub target lookup
 */
org.ametys.ribbon.control.button.ToolActionSelectionButton.prototype._subtargetRegExp;

org.ametys.ribbon.control.button.ToolActionSelectionButton.prototype.onMessage = function(messages)
{
	org.ametys.ribbon.control.button.ToolActionSelectionButton.superclass.onMessage.call(this, messages);

	for (var i=0; i < messages.length; i++)
	{
		if (messages[i].getType().getName() == org.ametys.ribbon.RibbonManager.EVENTTYPE_SELECTIONCHANGED)
		{
			var targets = messages[i].getTargets();
			var target = org.ametys.messagebus.message.MessageTargetHelper.findFirst(targets, this._testTarget.createDelegate(this));
			
			if (target != null)
			{
				this.getElements().each(function (element) { element.enable(); });
			}
			else
			{
				this.getElements().each(function (element) { element.disable(); });
			}
		}
	}
}

/**
 * @private
 * Test a target. Does it match ?
 * @return {org.ametys.messagebus.message.MessageTarget} The target to check
 */
org.ametys.ribbon.control.button.ToolActionSelectionButton.prototype._testTarget = function(target)
{
	return this._targetRegExp.test(target.getType()) 
		&& (this._subtargetRegExp == null || (target.getSubTargets().length > 0 && this._subtargetRegExp.test(target.getSubTargets()[0].getType()))); 
}
