/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.ribbon.control.button');

/**
 * Create the control instance.
 * @param {String} id The unique identifier
 * @param {Map<id, value>} configuration The configuration of the control instance.
 * @param {String} pluginName The name of the declaring plugin
 * @class This class implements the behavior for unique instance tool. These are toggle buttons showing hiding the tool.
 * @extends org.ametys.ribbon.control.RibbonControl
 */
org.ametys.ribbon.control.button.UniqueInstanceToolButton = function(id, configuration, pluginName)
{
	org.ametys.ribbon.control.button.UniqueInstanceToolButton.superclass.constructor.call(this, id, configuration, pluginName);
}

Ext.extend(org.ametys.ribbon.control.button.UniqueInstanceToolButton, org.ametys.ribbon.control.button.ToggleButton, {});

/**
 * @private
 * This handler is called when the toogle button state change
 */
org.ametys.ribbon.control.button.UniqueInstanceToolButton.prototype.onToggle = function(button, state)
{
	var control = org.ametys.ribbon.RibbonManager.getInstance().getControls()[this.controlId];
	if (state)
	{
		org.ametys.uitool.ToolsManager.getInstance().openTool(control.getConfiguration()['role'], control.getOpenToolParameters());
	}
	else
	{
		var tool = org.ametys.uitool.ToolsManager.getInstance().getTool(control.getConfiguration()['role']);
		
		if (tool.toolsPanel.getActiveTab() != tool.parent)
		{
			org.ametys.uitool.ToolsManager.getInstance().activateTool (tool, tool._factory._config['default-location']);
			this.toggle(true);
		}
		else
		{
			tool.close();
		}
	}
}

/**
 * Get the parameters needed by the factory to open the tool
 * @return {Map} params The parameters needed by the factory to open the tool. Can not be null.
 */
org.ametys.ribbon.control.button.UniqueInstanceToolButton.prototype.getOpenToolParameters = function()
{
	return {};
}

/**
 * When some messages arrived button may auto toggle
 * @param {org.ametys.messagebus.message.Message[]} messages The messages received.
 */
org.ametys.ribbon.control.button.UniqueInstanceToolButton.prototype.onMessage = function(messages)
{
	var shouldBeState = null;
	
	for (var i = 0; i < messages.length; i++)
	{
		var message = messages[i];
		
		switch (message.getType().getName())
		{
			case org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLOPENED:
				if (this.isInTarget(message.getTargets()))
				{
					shouldBeState = true;
				}
				break;
			case org.ametys.ribbon.RibbonManager.EVENTTYPE_TOOLCLOSED:
				if (this.isInTarget(message.getTargets()))
				{
					shouldBeState = false;
				}
				break;
		}
	}
	
	if (shouldBeState != null)
	{
		this.getElements().each(function (element) { element.toggle(shouldBeState, true, true); });
	}
}

/**
 * @private
 * This method returns true if current tool id is one of the target
 */
org.ametys.ribbon.control.button.UniqueInstanceToolButton.prototype.isInTarget = function(targets)
{
	for (var i = 0; i < targets.length; i++)
	{
		var target = targets[i];
		
		if (target.getType() == "tool" && target.getParameters()['id'] == this.getConfiguration()['role'])
		{
			return true;
		}
		
		if (this.isInTarget(target.getSubTargets()))
		{
			return true;
		}
	}
	
	return false;
}
