/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.uitool');

/**
 * Do not call this method. A unique instance of org.ametys.uitool.ToolsManager is used. Use org.ametys.uitool.ToolsManager.getInstance() instead. 
 * @constructor
 * @class This class has a single instance. Use org.ametys.uitool.ToolsManager.getInstance to get it.<br/>This class handle the tools. 
 */
org.ametys.uitool.ToolsManager = function ()
{
	this._tools = {};
	this._factories = {};
	this._autoRefreshFactory = {}
}

/**
 * @private
 * @property {org.ametys.uitool.ToolsManager} _instance Unique instance.
 */
org.ametys.uitool.ToolsManager._instance;


/**
 * This method returns the unique instance of the org.ametys.uitool.ToolsManager
 * @static
 * @returns {org.ametys.uitool.ToolsManager} The unique instance of the org.ametys.uitool.ToolsManager class.
 */
org.ametys.uitool.ToolsManager.getInstance = function()
{
	if (org.ametys.uitool.ToolsManager._instance == null)
	{
		org.ametys.uitool.ToolsManager._instance = new org.ametys.uitool.ToolsManager();		
	}
	return org.ametys.uitool.ToolsManager._instance;
}

/**
 * @private
 * @property _factories {@link org.ametys.uitool.tool.ToolFactory} in a Map<id, org.ametys.uitool.tool.ToolFactory>
 */
org.ametys.uitool.ToolsManager.prototype._factories;
/**
 * Add a new {@link org.ametys.uitool.tool.ToolFactory}
 * @param {String} role The role associated to the factory
 * @param {org.ametys.uitool.tool.ToolFactory} factory The factory
 */
org.ametys.uitool.ToolsManager.prototype.addFactory = function (role, factory)
{
	this._factories[role] = factory;
}


/**
 * @private
 * @property {Ext.Component} _panel The panel.
 */
org.ametys.uitool.ToolsManager._instance;

/**
 * @private
 * @property {Map<String, org.ametys.uitool.tool.Tool>} _tools The tools in a Map
 */
org.ametys.uitool.ToolsManager.prototype._tools;
/**
 * Get a tool by its id
 * @property {String} The id of the tool to get
 * @return {org.ametys.uitool.tool.Tool} The tool
 */
org.ametys.uitool.ToolsManager.prototype.getTool = function (id)
{
	return this._tools[id];
};
/**
 * Get the tools 
 * @return {Map<String, org.ametys.uitool.tool.Tool>} The tools in a Map
 */
org.ametys.uitool.ToolsManager.prototype.getTools = function ()
{
	return this._tools;
};
/**
 * Creates the panel and the center tool panel
 * @return {Ext.Component} The panel
 */
org.ametys.uitool.ToolsManager.prototype.createPanel = function ()
{
	this._panel = new org.ametys.uitool.tool.SimpleToolPanel ({
		id: 'tools-panel-wrapper',
		region: 'center', 
		html: ''
	});
	
	return this._panel;
};

/**
 * Ask to the factory to open the tool : the tool may be already existing (even already opened) or created now.<br/>This is the main method to call when you want to open a tool.
 * @param {String} role The tool to open
 * @param {Map} toolParams The needed parameters to open the tool
 */
org.ametys.uitool.ToolsManager.prototype.openTool  = function (role, toolParams)
{
	var factory = this._factories[role];
	var tool = factory.openTool (toolParams);
};


/**
 * This method activates the tab panel of the tool.
 * @param {org.ametys.uitool.tool.Tool} tool The tool to activate
 * @param {String} defaultLocation The default location
 */
org.ametys.uitool.ToolsManager.prototype.gotFocusOnActiveCenterTool = function ()
{
	// Force focus
	var tool = org.ametys.uitool.ToolsManager.getInstance().getTool(this._panel.items.get(0).uiTool);
	org.ametys.uitool.tool.ToolPanel.gotFocus (tool.parent);
}

/**
 * This method activates the tab panel of the tool.
 * @param {org.ametys.uitool.tool.Tool} tool The tool to activate
 * @param {String} defaultLocation The default location
 */
org.ametys.uitool.ToolsManager.prototype.activateTool  = function (tool, defaultLocation)
{
	// Nothing to do
}

/**
 * This method is called when the panel element (not the panel itself) may have received the focus.
 * The scope must be the panel.
 * @static
 * @private
 */
org.ametys.uitool.ToolsManager.prototype.gotElFocus = function(toolpanel)
{
	// Nothing to do
}

/**
 * Repaint the tooltip's panel.&lt;br/&gt;Use this method after changing title, description or icons.
 * @private
 */
org.ametys.uitool.ToolsManager.prototype.repaintTooltip = function (tool)
{
	// Nothing to do
};

/**
 * This method has to be used by a factory to really paint a just created tool. Use openTool instead. 
 * @param {org.ametys.uitool.tool.Tool} tool The tool to paint
 * @param {String} defaultLocation The default location
 */
org.ametys.uitool.ToolsManager.prototype.addUITool = function(tool, defaultLocation)
{
	this._tools[tool.getId()] =  tool;
	
	org.ametys.messagebus.MessageBus.getInstance().register(tool);
	
	this._panel.addUITool(tool);
	this._panel.doLayout();
};

/**
 * This method has to be used by a factory to really destroy a tool. Use Tool.close() instead. 
 * @param {org.ametys.uitool.tool.Tool} tool The tool to remove
 */
org.ametys.uitool.ToolsManager.prototype.removeUITool = function(tool)
{
	// Nothing to do
};

/**
 * @private
 * @property {Map<String, boolean>} _autoRefreshFactory The list of auto resreshing factories and the auto refresh associated value
 */
org.ametys.uitool.ToolsManager.prototype._autoRefreshFactory;

/**
 * Refresh the out dated tools
 * @param {String} factoryRole The concerned factory
 * @param {boolean} value True means this factory will auto refresh
 */
org.ametys.uitool.ToolsManager.prototype.setAutoRefresh = function(factoryRole, value)
{
	this._autoRefreshFactory[factoryRole] = value;
}

/**
 * Refresh the out dated tools
 */
org.ametys.uitool.ToolsManager.prototype.refreshTools = function ()
{	
	var tools = this.getTools();
	for (var i in tools)
	{
		var tool = tools[i];
		
		if (tool.isOutOfDate() && this._autoRefreshFactory[tool._factory._id])
		{
			tool.refresh();
		}
	}
}

org.ametys.uitool.ToolsManager.prototype._envParams = {};
org.ametys.uitool.ToolsManager.prototype.getEnvParams = function (paramName)
{
	return this._envParams[paramName];
}

org.ametys.uitool.ToolsManager.prototype.setEnvParams = function (params)
{
	this._envParams = params;
}

org.ametys.uitool.ToolsManager.prototype.exitCMS = function ()
{
	if (this.getEnvParams('back-url') != null)
	{
		window.location.href = this.getEnvParams('back-url');
	}
	if (this.getEnvParams('callback') != null)
	{
		eval('window.parent.' + this.getEnvParams('callback') + '()');
	}
	else
	{
		window.close();
	}
}