/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.uitool.tool.impl');


/**
 * Creates an instance of org.ametys.uitool.tool.impl.UniqueInstanceToolFactory
 * @class This class represents a factory for an unique tool<br/>Extend this class if your tool should be an unique instance (no more than one tool opened) and override the {@link org.ametys.uitool.tool.UniqueInstanceToolFactory.#createTool} method.
 * @extends org.ametys.uitool.tool.ToolFactory
 * @param {String} id The unique id of the factory
 * @param {Map} config Configuration options 
 * @param {String} pluginName The name of the plugin
 * @constructor
 */
org.ametys.uitool.tool.impl.UniqueInstanceToolFactory = function(id, config, pluginName) 
{
	org.ametys.uitool.tool.impl.UniqueInstanceToolFactory.superclass.constructor.call(this, id, config, pluginName);
};

Ext.extend(org.ametys.uitool.tool.impl.UniqueInstanceToolFactory, org.ametys.uitool.tool.ToolFactory,
{
});

/**
 * @private
 * @property {org.ametys.uitool.tool.Tool} _tool The tool associated with this factory
 */
org.ametys.uitool.tool.impl.UniqueInstanceToolFactory.prototype._tool;

/**
 * Inherit this method to create your own tool
 * @param {String} id The unique id of the new tool.
 * @return {org.ametys.uitool.tool.Tool} The tool to open.
 */
org.ametys.uitool.tool.impl.UniqueInstanceToolFactory.prototype.createTool = function (id)
{
	throw "org.ametys.uitool.tool.impl.UniqueInstanceToolFactory#createTool is not implemented";
}

/**
 * Inherit this method in your own factory.<br/>The purpose of this method is to open the tool to make it visible in the interface.
 * @param {Map} params The parameters needed by the factory to open the tool. Can not be null.
 * @return {org.ametys.uitool.tool.Tool} The created tool.
 */
org.ametys.uitool.tool.impl.UniqueInstanceToolFactory.prototype.openTool = function(params) 
{
	// We can use the factory id, because this factory will handle only one tool.
	if (org.ametys.uitool.ToolsManager.getInstance().getTool(this._id) == null)
	{
		if (this._tool == null)
		{
			// Tool does not exists => Tool creation.
			// We can use the factory id, because this factory will handle only one tool.
			this._tool = this.createTool(this._id);
			this._tool.setFactory (this); 
			this._tool.setConfiguration(this._config);
		}
		
		org.ametys.uitool.ToolsManager.getInstance().addUITool (this._tool, this._config['default-location']);
	}	

	this._tool.setParams(params);
	org.ametys.uitool.ToolsManager.getInstance().activateTool (this._tool, this._config['default-location']);
	
	return this._tool;
}

/**
 * Close a tool.
 * @param {org.ametys.uitool.tool.Tool} tool The tool to close.
 */
org.ametys.uitool.tool.impl.UniqueInstanceToolFactory.prototype.closeTool = function(tool) 
{
	org.ametys.uitool.ToolsManager.getInstance().removeUITool (this._tool);
	this._tool = null;
}